/*

  silcasync.h

  Author: Pekka Riikonen <priikone@silcnet.org>

  Copyright (C) 2005 Pekka Riikonen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

*/

/****h* silcutil/SILC Async Operation Interface
 *
 * DESCRIPTION
 *
 * SILC Async Operation API is an interface that can be used to control
 * asynchronous operations.  All functions that take callback as argument
 * should return SilcAsyncOperation context.  That context then can be
 * used to control, such as, abort the asynchronous operation.  Using
 * SILC Async Operation API, asynchronous functions can be controlled
 * and aborted safely.
 *
 ***/

#ifndef SILCASYNC_H
#define SILCASYNC_H

/****s* silcutil/SilcAsyncOperationAPI/SilcStream
 *
 * NAME
 *
 *    typedef struct SilcAsyncOperationStruct *SilcAsyncOperation;
 *
 * DESCRIPTION
 *
 *    Abstact stream context representing any stream.  All streams are using
 *    this abstraction so that the stream can be accessed using the standard
 *    silc_stream_* functions.  All streams are destroyed by calling the
 *    silc_stream_destroy function.
 *
 ***/
typedef struct SilcAsyncOperationStruct *SilcAsyncOperation;

/****f* silcutil/SilcAsyncOperationAPI/SilcAsyncOperationAbort
 *
 * SYNOPSIS
 *
 *    typedef void (*SilcAsyncOperationAbort)(SilcAsyncOperation op,
 *                                            void *context);
 *
 * DESCRIPTION
 *
 *    This callback is called when upper layer calls the silc_async_abort,
 *    and is used to actually perform the abortion of the asynchronous
 *    operation.  The silc_async_stop must not be called in this function.
 *
 *    This callback type can also be provided to silc_async_abort function
 *    by the upper layer, if it wants that callback is called to the upper
 *    layer when aborting the operation.
 *
 ***/
typedef void (*SilcAsyncOperationAbort)(SilcAsyncOperation op,
					void *context);

/****f* silcutil/SilcAsyncOperationAPI/SilcAsyncOperationPause
 *
 * SYNOPSIS
 *
 *    typedef void (*SilcAsyncOperationPause)(SilcAsyncOperation op,
 *                                            bool pause_operation,
 *                                            void *context);
 *
 * DESCRIPTION
 *
 *    This callback is used to halt an operation, if upper layer calls the
 *    silc_async_halt function, or to resume an operation if upper layer
 *    calls the silc_async_resume, after it has earlier halted the operation.
 *    If this callback is implemented it is guaranteed that the asynchronous
 *    operation is not progressed when it is halted. If the `pause_operation'
 *    is TRUE the operation is halted.  If it is FALSE, then the operation
 *    resumes its execution.
 *
 ***/
typedef void (*SilcAsyncOperationPause)(SilcAsyncOperation op,
                                        bool pause_operation,
				        void *context);

/* Upper layer functions for managing asynchronous operations.  Layer
   that has received SilcAsyncOperation context can control the async
   operation with these functions. */

/****f* silcutil/SilcAsyncOperationAPI/silc_async_halt
 *
 * SYNOPSIS
 *
 *    void silc_async_halt(SilcAsyncOperation op);
 *
 * DESCRIPTION
 *
 *    Halt the execution of the asynchronous operation.  If the operation
 *    supports this feature, it is guaranteed that the operation is halted
 *    and its execution is not progressed until the silc_async_resume function
 *    is called.  The operation still can be aborted even if it is halted.
 *    If this function is not supported, calling this has no effect.
 *
 ***/
void silc_async_halt(SilcAsyncOperation op);

/****f* silcutil/SilcAsyncOperationAPI/silc_async_resume
 *
 * SYNOPSIS
 *
 *    void silc_async_resume(SilcAsyncOperation op);
 *
 * DESCRIPTION
 *
 *    Resume the execution of the asynchronous operation.  If the halting of
 *    the operation was supported, then this function is used to resume the
 *    execution of the operation after it was halted.  If this function is
 *    not supported, calling this has no effect.
 *
 ***/
void silc_async_resume(SilcAsyncOperation op);

/****f* silcutil/SilcAsyncOperationAPI/silc_async_abort
 *
 * SYNOPSIS
 *
 *    void silc_async_abort(SilcAsyncOperation op,
 *                          SilcAsyncOperationAbort abort, void *context);
 *
 * DESCRIPTION
 *
 *    This function is used by the layer who received SilcAsyncOperation
 *    context from an callback function to abort the asynchronous operation.
 *    The `op' becomes invalid after this function returns.  It is also
 *    guaranteed (assuming the use of this API is implemented correctly) that
 *    some other completion callback is not called after the operation was
 *    aborted.  However, if the caller wants to receive a callback when
 *    aborting the caller may specify the `abort' and `context' which will
 *    be called after the operation is aborted, but before the `op' becomes
 *    invalid.  The `abort' is called immediately inside this function.
 *
 ***/
void silc_async_abort(SilcAsyncOperation op,
                      SilcAsyncOperationAbort abort, void *context);

/* The operation layer functions.  The layer that performs the async
   operation use these functions. */

/****f* silcutil/SilcAsyncOperationAPI/silc_async_start
 *
 * SYNOPSIS
 *
 *    SilcAsyncOperation silc_async_start(SilcAsyncOperationAbort abort,
 *                                        SilcAsyncOperationPause pause,
 *                                        void *context);
 *
 * DESCRIPTION
 *
 *    Start asynchronous operation, and assign `abort' callback for it,
 *    which can be used by some upper layer to abort the asynchronous
 *    operation, by calling the silc_async_abort.  The layer which calls
 *    this function must also call silc_async_stop when the asynchronous
 *    operation is successfully completed.  If it is aborted by upper layer
 *    then silc_async_stop must not be called, since it is called by the
 *    silc_async_abort function.
 *
 *    If the `pause' is provided then the upper layer may also halt and
 *    then later resume the execution of the operation, by calling the
 *    silc_async_halt and silc_async_resume respectively.  If `pause' is
 *    not provided then these functions has no effect for this operation.
 *
 ***/
SilcAsyncOperation silc_async_start(SilcAsyncOperationAbort abort,
				    SilcAsyncOperationPause pause,
				    void *context);

/****f* silcutil/SilcAsyncOperationAPI/silc_async_stop
 *
 * SYNOPSIS
 *
 *    void silc_async_stop(SilcAsyncOperation op);
 *
 * DESCRIPTION
 *
 *    Stop the asynchronous operation.  If the asynchronous operation ended
 *    normally (ie. it was not aborted) this function must be called by the
 *    caller who called silc_async_start.  The `op' will become invalid after
 *    this and the upper layer must not call silc_async_abort after this
 *    function is called.  The layer that calls this, must call some other
 *    completion callback to the upper layer, so that it knows that the
 *    asynchronous operation is completed.
 *
 ***/
void silc_async_stop(SilcAsyncOperation op);

#endif /* SILCASYNC_H */
